# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    MessagingHostedNumberOrderListResponse,
    MessagingHostedNumberOrderCreateResponse,
    MessagingHostedNumberOrderDeleteResponse,
    MessagingHostedNumberOrderRetrieveResponse,
    MessagingHostedNumberOrderValidateCodesResponse,
    MessagingHostedNumberOrderCheckEligibilityResponse,
    MessagingHostedNumberOrderCreateVerificationCodesResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMessagingHostedNumberOrders:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        messaging_hosted_number_order = client.messaging_hosted_number_orders.create()
        assert_matches_type(MessagingHostedNumberOrderCreateResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        messaging_hosted_number_order = client.messaging_hosted_number_orders.create(
            messaging_profile_id="dc8f39ac-953d-4520-b93b-786ae87db0da",
            phone_numbers=["+18665550001", "+18665550002"],
        )
        assert_matches_type(MessagingHostedNumberOrderCreateResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.messaging_hosted_number_orders.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_hosted_number_order = response.parse()
        assert_matches_type(MessagingHostedNumberOrderCreateResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.messaging_hosted_number_orders.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_hosted_number_order = response.parse()
            assert_matches_type(
                MessagingHostedNumberOrderCreateResponse, messaging_hosted_number_order, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        messaging_hosted_number_order = client.messaging_hosted_number_orders.retrieve(
            "id",
        )
        assert_matches_type(
            MessagingHostedNumberOrderRetrieveResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.messaging_hosted_number_orders.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_hosted_number_order = response.parse()
        assert_matches_type(
            MessagingHostedNumberOrderRetrieveResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.messaging_hosted_number_orders.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_hosted_number_order = response.parse()
            assert_matches_type(
                MessagingHostedNumberOrderRetrieveResponse, messaging_hosted_number_order, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.messaging_hosted_number_orders.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        messaging_hosted_number_order = client.messaging_hosted_number_orders.list()
        assert_matches_type(MessagingHostedNumberOrderListResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        messaging_hosted_number_order = client.messaging_hosted_number_orders.list(
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(MessagingHostedNumberOrderListResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.messaging_hosted_number_orders.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_hosted_number_order = response.parse()
        assert_matches_type(MessagingHostedNumberOrderListResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.messaging_hosted_number_orders.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_hosted_number_order = response.parse()
            assert_matches_type(
                MessagingHostedNumberOrderListResponse, messaging_hosted_number_order, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        messaging_hosted_number_order = client.messaging_hosted_number_orders.delete(
            "id",
        )
        assert_matches_type(MessagingHostedNumberOrderDeleteResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.messaging_hosted_number_orders.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_hosted_number_order = response.parse()
        assert_matches_type(MessagingHostedNumberOrderDeleteResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.messaging_hosted_number_orders.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_hosted_number_order = response.parse()
            assert_matches_type(
                MessagingHostedNumberOrderDeleteResponse, messaging_hosted_number_order, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.messaging_hosted_number_orders.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_check_eligibility(self, client: Telnyx) -> None:
        messaging_hosted_number_order = client.messaging_hosted_number_orders.check_eligibility(
            phone_numbers=["string"],
        )
        assert_matches_type(
            MessagingHostedNumberOrderCheckEligibilityResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_check_eligibility(self, client: Telnyx) -> None:
        response = client.messaging_hosted_number_orders.with_raw_response.check_eligibility(
            phone_numbers=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_hosted_number_order = response.parse()
        assert_matches_type(
            MessagingHostedNumberOrderCheckEligibilityResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_check_eligibility(self, client: Telnyx) -> None:
        with client.messaging_hosted_number_orders.with_streaming_response.check_eligibility(
            phone_numbers=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_hosted_number_order = response.parse()
            assert_matches_type(
                MessagingHostedNumberOrderCheckEligibilityResponse, messaging_hosted_number_order, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_verification_codes(self, client: Telnyx) -> None:
        messaging_hosted_number_order = client.messaging_hosted_number_orders.create_verification_codes(
            id="id",
            phone_numbers=["string"],
            verification_method="sms",
        )
        assert_matches_type(
            MessagingHostedNumberOrderCreateVerificationCodesResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create_verification_codes(self, client: Telnyx) -> None:
        response = client.messaging_hosted_number_orders.with_raw_response.create_verification_codes(
            id="id",
            phone_numbers=["string"],
            verification_method="sms",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_hosted_number_order = response.parse()
        assert_matches_type(
            MessagingHostedNumberOrderCreateVerificationCodesResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create_verification_codes(self, client: Telnyx) -> None:
        with client.messaging_hosted_number_orders.with_streaming_response.create_verification_codes(
            id="id",
            phone_numbers=["string"],
            verification_method="sms",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_hosted_number_order = response.parse()
            assert_matches_type(
                MessagingHostedNumberOrderCreateVerificationCodesResponse,
                messaging_hosted_number_order,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create_verification_codes(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.messaging_hosted_number_orders.with_raw_response.create_verification_codes(
                id="",
                phone_numbers=["string"],
                verification_method="sms",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_validate_codes(self, client: Telnyx) -> None:
        messaging_hosted_number_order = client.messaging_hosted_number_orders.validate_codes(
            id="id",
            verification_codes=[
                {
                    "code": "code",
                    "phone_number": "phone_number",
                }
            ],
        )
        assert_matches_type(
            MessagingHostedNumberOrderValidateCodesResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_validate_codes(self, client: Telnyx) -> None:
        response = client.messaging_hosted_number_orders.with_raw_response.validate_codes(
            id="id",
            verification_codes=[
                {
                    "code": "code",
                    "phone_number": "phone_number",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_hosted_number_order = response.parse()
        assert_matches_type(
            MessagingHostedNumberOrderValidateCodesResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_validate_codes(self, client: Telnyx) -> None:
        with client.messaging_hosted_number_orders.with_streaming_response.validate_codes(
            id="id",
            verification_codes=[
                {
                    "code": "code",
                    "phone_number": "phone_number",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_hosted_number_order = response.parse()
            assert_matches_type(
                MessagingHostedNumberOrderValidateCodesResponse, messaging_hosted_number_order, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_validate_codes(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.messaging_hosted_number_orders.with_raw_response.validate_codes(
                id="",
                verification_codes=[
                    {
                        "code": "code",
                        "phone_number": "phone_number",
                    }
                ],
            )


class TestAsyncMessagingHostedNumberOrders:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        messaging_hosted_number_order = await async_client.messaging_hosted_number_orders.create()
        assert_matches_type(MessagingHostedNumberOrderCreateResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        messaging_hosted_number_order = await async_client.messaging_hosted_number_orders.create(
            messaging_profile_id="dc8f39ac-953d-4520-b93b-786ae87db0da",
            phone_numbers=["+18665550001", "+18665550002"],
        )
        assert_matches_type(MessagingHostedNumberOrderCreateResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_hosted_number_orders.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_hosted_number_order = await response.parse()
        assert_matches_type(MessagingHostedNumberOrderCreateResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_hosted_number_orders.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_hosted_number_order = await response.parse()
            assert_matches_type(
                MessagingHostedNumberOrderCreateResponse, messaging_hosted_number_order, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        messaging_hosted_number_order = await async_client.messaging_hosted_number_orders.retrieve(
            "id",
        )
        assert_matches_type(
            MessagingHostedNumberOrderRetrieveResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_hosted_number_orders.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_hosted_number_order = await response.parse()
        assert_matches_type(
            MessagingHostedNumberOrderRetrieveResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_hosted_number_orders.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_hosted_number_order = await response.parse()
            assert_matches_type(
                MessagingHostedNumberOrderRetrieveResponse, messaging_hosted_number_order, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.messaging_hosted_number_orders.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        messaging_hosted_number_order = await async_client.messaging_hosted_number_orders.list()
        assert_matches_type(MessagingHostedNumberOrderListResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        messaging_hosted_number_order = await async_client.messaging_hosted_number_orders.list(
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(MessagingHostedNumberOrderListResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_hosted_number_orders.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_hosted_number_order = await response.parse()
        assert_matches_type(MessagingHostedNumberOrderListResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_hosted_number_orders.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_hosted_number_order = await response.parse()
            assert_matches_type(
                MessagingHostedNumberOrderListResponse, messaging_hosted_number_order, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        messaging_hosted_number_order = await async_client.messaging_hosted_number_orders.delete(
            "id",
        )
        assert_matches_type(MessagingHostedNumberOrderDeleteResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_hosted_number_orders.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_hosted_number_order = await response.parse()
        assert_matches_type(MessagingHostedNumberOrderDeleteResponse, messaging_hosted_number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_hosted_number_orders.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_hosted_number_order = await response.parse()
            assert_matches_type(
                MessagingHostedNumberOrderDeleteResponse, messaging_hosted_number_order, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.messaging_hosted_number_orders.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_check_eligibility(self, async_client: AsyncTelnyx) -> None:
        messaging_hosted_number_order = await async_client.messaging_hosted_number_orders.check_eligibility(
            phone_numbers=["string"],
        )
        assert_matches_type(
            MessagingHostedNumberOrderCheckEligibilityResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_check_eligibility(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_hosted_number_orders.with_raw_response.check_eligibility(
            phone_numbers=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_hosted_number_order = await response.parse()
        assert_matches_type(
            MessagingHostedNumberOrderCheckEligibilityResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_check_eligibility(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_hosted_number_orders.with_streaming_response.check_eligibility(
            phone_numbers=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_hosted_number_order = await response.parse()
            assert_matches_type(
                MessagingHostedNumberOrderCheckEligibilityResponse, messaging_hosted_number_order, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_verification_codes(self, async_client: AsyncTelnyx) -> None:
        messaging_hosted_number_order = await async_client.messaging_hosted_number_orders.create_verification_codes(
            id="id",
            phone_numbers=["string"],
            verification_method="sms",
        )
        assert_matches_type(
            MessagingHostedNumberOrderCreateVerificationCodesResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create_verification_codes(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_hosted_number_orders.with_raw_response.create_verification_codes(
            id="id",
            phone_numbers=["string"],
            verification_method="sms",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_hosted_number_order = await response.parse()
        assert_matches_type(
            MessagingHostedNumberOrderCreateVerificationCodesResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create_verification_codes(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_hosted_number_orders.with_streaming_response.create_verification_codes(
            id="id",
            phone_numbers=["string"],
            verification_method="sms",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_hosted_number_order = await response.parse()
            assert_matches_type(
                MessagingHostedNumberOrderCreateVerificationCodesResponse,
                messaging_hosted_number_order,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create_verification_codes(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.messaging_hosted_number_orders.with_raw_response.create_verification_codes(
                id="",
                phone_numbers=["string"],
                verification_method="sms",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_validate_codes(self, async_client: AsyncTelnyx) -> None:
        messaging_hosted_number_order = await async_client.messaging_hosted_number_orders.validate_codes(
            id="id",
            verification_codes=[
                {
                    "code": "code",
                    "phone_number": "phone_number",
                }
            ],
        )
        assert_matches_type(
            MessagingHostedNumberOrderValidateCodesResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_validate_codes(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_hosted_number_orders.with_raw_response.validate_codes(
            id="id",
            verification_codes=[
                {
                    "code": "code",
                    "phone_number": "phone_number",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_hosted_number_order = await response.parse()
        assert_matches_type(
            MessagingHostedNumberOrderValidateCodesResponse, messaging_hosted_number_order, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_validate_codes(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_hosted_number_orders.with_streaming_response.validate_codes(
            id="id",
            verification_codes=[
                {
                    "code": "code",
                    "phone_number": "phone_number",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_hosted_number_order = await response.parse()
            assert_matches_type(
                MessagingHostedNumberOrderValidateCodesResponse, messaging_hosted_number_order, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_validate_codes(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.messaging_hosted_number_orders.with_raw_response.validate_codes(
                id="",
                verification_codes=[
                    {
                        "code": "code",
                        "phone_number": "phone_number",
                    }
                ],
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .global_ip_assignment import GlobalIPAssignment

__all__ = ["GlobalIPAssignmentUpdateResponse"]


class GlobalIPAssignmentUpdateResponse(BaseModel):
    data: Optional[GlobalIPAssignment] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from .record import Record
from .._models import BaseModel

__all__ = ["GlobalIPCreateResponse", "Data"]


class Data(Record):
    description: Optional[str] = None
    """A user specified description for the address."""

    ip_address: Optional[str] = None
    """The Global IP address."""

    name: Optional[str] = None
    """A user specified name for the address."""

    ports: Optional[Dict[str, object]] = None
    """A Global IP ports grouped by protocol code."""

    record_type: Optional[str] = None  # type: ignore
    """Identifies the type of the resource."""


class GlobalIPCreateResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .sub_number_order import SubNumberOrder

__all__ = ["SubNumberOrderCancelResponse"]


class SubNumberOrderCancelResponse(BaseModel):
    data: Optional[SubNumberOrder] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .conference import Conference
from .pagination_meta import PaginationMeta

__all__ = ["ConferenceListResponse"]


class ConferenceListResponse(BaseModel):
    data: Optional[List[Conference]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["RecordingListParams", "Filter", "FilterCreatedAt", "Page"]


class RecordingListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[conference_id], filter[created_at][gte],
    filter[created_at][lte], filter[call_leg_id], filter[call_session_id],
    filter[from], filter[to], filter[connection_id]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """


class FilterCreatedAt(TypedDict, total=False):
    gte: str
    """Returns only recordings created later than or at given ISO 8601 datetime."""

    lte: str
    """Returns only recordings created earlier than or at given ISO 8601 datetime."""


_FilterReservedKeywords = TypedDict(
    "_FilterReservedKeywords",
    {
        "from": str,
    },
    total=False,
)


class Filter(_FilterReservedKeywords, total=False):
    call_leg_id: str
    """If present, recordings will be filtered to those with a matching call_leg_id."""

    call_session_id: str
    """
    If present, recordings will be filtered to those with a matching
    call_session_id.
    """

    conference_id: str
    """Returns only recordings associated with a given conference."""

    connection_id: str
    """
    If present, recordings will be filtered to those with a matching `connection_id`
    attribute (case-sensitive).
    """

    created_at: FilterCreatedAt

    to: str
    """
    If present, recordings will be filtered to those with a matching `to` attribute
    (case-sensitive).
    """


class Page(TypedDict, total=False):
    number: int
    """The page number to load."""

    size: int
    """The size of the page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel
from .insight_template import InsightTemplate
from ..assistants.tests.test_suites.meta import Meta

__all__ = ["InsightListResponse"]


class InsightListResponse(BaseModel):
    data: List[InsightTemplate]

    meta: Meta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["SimCardOrder", "Cost", "OrderAddress"]


class Cost(BaseModel):
    amount: Optional[str] = None
    """A string representing the cost amount."""

    currency: Optional[str] = None
    """Filter by ISO 4217 currency string."""


class OrderAddress(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the address for the order."""

    administrative_area: Optional[str] = None
    """State or province where the address is located."""

    business_name: Optional[str] = None
    """The name of the business where the address is located."""

    country_code: Optional[str] = None
    """The mobile operator two-character (ISO 3166-1 alpha-2) origin country code."""

    extended_address: Optional[str] = None
    """Supplemental field for address information."""

    first_name: Optional[str] = None
    """The first name of the shipping recipient."""

    last_name: Optional[str] = None
    """The last name of the shipping recipient."""

    locality: Optional[str] = None
    """The name of the city where the address is located."""

    postal_code: Optional[str] = None
    """Postal code for the address."""

    street_address: Optional[str] = None
    """The name of the street where the address is located."""


class SimCardOrder(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    cost: Optional[Cost] = None
    """An object representing the total cost of the order."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was last created."""

    order_address: Optional[OrderAddress] = None
    """
    An object representing the address information from when the order was
    submitted.
    """

    quantity: Optional[int] = None
    """The amount of SIM cards requested in the SIM card order."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    status: Optional[Literal["pending", "processing", "ready_to_ship", "shipped", "delivered", "canceled"]] = None
    """
    The current status of the SIM Card order.<ul> <li><code>pending</code> - the
    order is waiting to be processed.</li> <li><code>processing</code> - the order
    is currently being processed.</li> <li><code>ready_to_ship</code> - the order is
    ready to be shipped to the specified <b>address</b>.</li>
    <li><code>shipped</code> - the order was shipped and is on its way to be
    delivered to the specified <b>address</b>.</li> <li><code>delivered</code> - the
    order was delivered to the specified <b>address</b>.</li>
    <li><code>canceled</code> - the order was canceled.</li> </ul>
    """

    tracking_url: Optional[str] = None
    """The URL used to get tracking information about the order."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["AzureConfigurationDataParam"]


class AzureConfigurationDataParam(TypedDict, total=False):
    account_key: str
    """Azure Blob Storage account key."""

    account_name: str
    """Azure Blob Storage account name."""

    bucket: str
    """Name of the bucket to be used to store recording files."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from ..pagination_meta import PaginationMeta

__all__ = ["CommentListResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None

    body: Optional[str] = None
    """Body of comment"""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    porting_order_id: Optional[str] = None

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    user_type: Optional[Literal["admin", "user", "system"]] = None
    """Indicates whether this comment was created by a Telnyx Admin, user, or system"""


class CommentListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .record import Record
from .._models import BaseModel
from .interface import Interface
from .region_in import RegionIn
from .pagination_meta import PaginationMeta

__all__ = ["VirtualCrossConnectListResponse", "Data", "DataRegion"]


class DataRegion(BaseModel):
    code: Optional[str] = None
    """Region code of the interface."""

    name: Optional[str] = None
    """Region name of the interface."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""


class Data(Record, Interface, RegionIn):
    bgp_asn: float
    """The Border Gateway Protocol (BGP) Autonomous System Number (ASN).

    If null, value will be assigned by Telnyx.
    """

    cloud_provider: Literal["aws", "azure", "gce"]
    """
    The Virtual Private Cloud with which you would like to establish a cross
    connect.
    """

    cloud_provider_region: str
    """
    The region where your Virtual Private Cloud hosts are located.<br /><br />The
    available regions can be found using the /virtual_cross_connect_regions
    endpoint.
    """

    primary_cloud_account_id: str
    """The identifier for your Virtual Private Cloud.

    The number will be different based upon your Cloud provider.
    """

    region_code: str  # type: ignore
    """The region interface is deployed to."""

    bandwidth_mbps: Optional[float] = None
    """
    The desired throughput in Megabits per Second (Mbps) for your Virtual Cross
    Connect.<br /><br />The available bandwidths can be found using the
    /virtual_cross_connect_regions endpoint.
    """

    primary_bgp_key: Optional[str] = None
    """The authentication key for BGP peer configuration."""

    primary_cloud_ip: Optional[str] = None
    """
    The IP address assigned for your side of the Virtual Cross
    Connect.<br /><br />If none is provided, one will be generated for
    you.<br /><br />This value can not be patched once the VXC has bene provisioned.
    """

    primary_enabled: Optional[bool] = None
    """Indicates whether the primary circuit is enabled.

    Setting this to `false` will disable the circuit.
    """

    primary_routing_announcement: Optional[bool] = None
    """Whether the primary BGP route is being announced."""

    primary_telnyx_ip: Optional[str] = None
    """
    The IP address assigned to the Telnyx side of the Virtual Cross
    Connect.<br /><br />If none is provided, one will be generated for
    you.<br /><br />This value should be null for GCE as Google will only inform you
    of your assigned IP once the connection has been accepted.
    """

    record_type: Optional[str] = None  # type: ignore
    """Identifies the type of the resource."""

    region: Optional[DataRegion] = None

    secondary_bgp_key: Optional[str] = None
    """The authentication key for BGP peer configuration."""

    secondary_cloud_account_id: Optional[str] = None
    """The identifier for your Virtual Private Cloud.

    The number will be different based upon your Cloud provider.<br /><br />This
    attribute is only necessary for GCE.
    """

    secondary_cloud_ip: Optional[str] = None
    """
    The IP address assigned for your side of the Virtual Cross
    Connect.<br /><br />If none is provided, one will be generated for
    you.<br /><br />This value can not be patched once the VXC has bene provisioned.
    """

    secondary_enabled: Optional[bool] = None
    """Indicates whether the secondary circuit is enabled.

    Setting this to `false` will disable the circuit.
    """

    secondary_routing_announcement: Optional[bool] = None
    """Whether the secondary BGP route is being announced."""

    secondary_telnyx_ip: Optional[str] = None
    """
    The IP address assigned to the Telnyx side of the Virtual Cross
    Connect.<br /><br />If none is provided, one will be generated for
    you.<br /><br />This value should be null for GCE as Google will only inform you
    of your assigned IP once the connection has been accepted.
    """


class VirtualCrossConnectListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .interface_status import InterfaceStatus

__all__ = ["Interface"]


class Interface(BaseModel):
    name: Optional[str] = None
    """A user specified name for the interface."""

    network_id: Optional[str] = None
    """The id of the network associated with the interface."""

    status: Optional[InterfaceStatus] = None
    """The current status of the interface deployment."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel

__all__ = [
    "MessagingHostedNumberOrderCreateVerificationCodesResponse",
    "Data",
    "DataVerificationCodeSuccess",
    "DataVerificationCodeError",
]


class DataVerificationCodeSuccess(BaseModel):
    phone_number: str
    """Phone number for which the verification code was created"""

    type: Literal["sms", "call", "flashcall"]
    """Type of verification method used"""

    verification_code_id: str
    """Unique identifier for the verification code"""


class DataVerificationCodeError(BaseModel):
    error: str
    """Error message describing why the verification code creation failed"""

    phone_number: str
    """Phone number for which the verification code creation failed"""


Data: TypeAlias = Union[DataVerificationCodeSuccess, DataVerificationCodeError]


class MessagingHostedNumberOrderCreateVerificationCodesResponse(BaseModel):
    data: List[Data]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.storage import migration_create_params
from ....types.storage.migration_list_response import MigrationListResponse
from ....types.storage.migration_create_response import MigrationCreateResponse
from ....types.storage.migration_retrieve_response import MigrationRetrieveResponse

__all__ = ["MigrationsResource", "AsyncMigrationsResource"]


class MigrationsResource(SyncAPIResource):
    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> MigrationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MigrationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MigrationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MigrationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        source_id: str,
        target_bucket_name: str,
        target_region: str,
        refresh: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MigrationCreateResponse:
        """Initiate a migration of data from an external provider into Telnyx Cloud
        Storage.

        Currently, only S3 is supported.

        Args:
          source_id: ID of the Migration Source from which to migrate data.

          target_bucket_name: Bucket name to migrate the data into. Will default to the same name as the
              `source_bucket_name`.

          target_region: Telnyx Cloud Storage region to migrate the data to.

          refresh: If true, will continue to poll the source bucket to ensure new data is
              continually migrated over.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/storage/migrations",
            body=maybe_transform(
                {
                    "source_id": source_id,
                    "target_bucket_name": target_bucket_name,
                    "target_region": target_region,
                    "refresh": refresh,
                },
                migration_create_params.MigrationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MigrationCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MigrationRetrieveResponse:
        """
        Get a Migration

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/storage/migrations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MigrationRetrieveResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MigrationListResponse:
        """List all Migrations"""
        return self._get(
            "/storage/migrations",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MigrationListResponse,
        )


class AsyncMigrationsResource(AsyncAPIResource):
    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMigrationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMigrationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMigrationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMigrationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        source_id: str,
        target_bucket_name: str,
        target_region: str,
        refresh: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MigrationCreateResponse:
        """Initiate a migration of data from an external provider into Telnyx Cloud
        Storage.

        Currently, only S3 is supported.

        Args:
          source_id: ID of the Migration Source from which to migrate data.

          target_bucket_name: Bucket name to migrate the data into. Will default to the same name as the
              `source_bucket_name`.

          target_region: Telnyx Cloud Storage region to migrate the data to.

          refresh: If true, will continue to poll the source bucket to ensure new data is
              continually migrated over.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/storage/migrations",
            body=await async_maybe_transform(
                {
                    "source_id": source_id,
                    "target_bucket_name": target_bucket_name,
                    "target_region": target_region,
                    "refresh": refresh,
                },
                migration_create_params.MigrationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MigrationCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MigrationRetrieveResponse:
        """
        Get a Migration

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/storage/migrations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MigrationRetrieveResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MigrationListResponse:
        """List all Migrations"""
        return await self._get(
            "/storage/migrations",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MigrationListResponse,
        )


class MigrationsResourceWithRawResponse:
    def __init__(self, migrations: MigrationsResource) -> None:
        self._migrations = migrations

        self.create = to_raw_response_wrapper(
            migrations.create,
        )
        self.retrieve = to_raw_response_wrapper(
            migrations.retrieve,
        )
        self.list = to_raw_response_wrapper(
            migrations.list,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._migrations.actions)


class AsyncMigrationsResourceWithRawResponse:
    def __init__(self, migrations: AsyncMigrationsResource) -> None:
        self._migrations = migrations

        self.create = async_to_raw_response_wrapper(
            migrations.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            migrations.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            migrations.list,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._migrations.actions)


class MigrationsResourceWithStreamingResponse:
    def __init__(self, migrations: MigrationsResource) -> None:
        self._migrations = migrations

        self.create = to_streamed_response_wrapper(
            migrations.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            migrations.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            migrations.list,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._migrations.actions)


class AsyncMigrationsResourceWithStreamingResponse:
    def __init__(self, migrations: AsyncMigrationsResource) -> None:
        self._migrations = migrations

        self.create = async_to_streamed_response_wrapper(
            migrations.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            migrations.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            migrations.list,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._migrations.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .pagination_response import PaginationResponse
from .billing_bundle_summary import BillingBundleSummary

__all__ = ["BillingBundleListResponse"]


class BillingBundleListResponse(BaseModel):
    data: List[BillingBundleSummary]

    meta: PaginationResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["TelephonyCredentialListParams", "Filter", "Page"]


class TelephonyCredentialListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[tag], filter[name], filter[status], filter[resource_id],
    filter[sip_username]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """


class Filter(TypedDict, total=False):
    name: str
    """Filter by name"""

    resource_id: str
    """Filter by resource_id"""

    sip_username: str
    """Filter by sip_username"""

    status: str
    """Filter by status"""

    tag: str
    """Filter by tag"""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast
from typing_extensions import Literal

import httpx

from ...types import fax_list_params, fax_create_params
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.fax_list_response import FaxListResponse
from ...types.fax_create_response import FaxCreateResponse
from ...types.fax_retrieve_response import FaxRetrieveResponse

__all__ = ["FaxesResource", "AsyncFaxesResource"]


class FaxesResource(SyncAPIResource):
    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> FaxesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return FaxesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FaxesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return FaxesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        connection_id: str,
        from_: str,
        to: str,
        client_state: str | Omit = omit,
        from_display_name: str | Omit = omit,
        media_name: str | Omit = omit,
        media_url: str | Omit = omit,
        monochrome: bool | Omit = omit,
        preview_format: Literal["pdf", "tiff"] | Omit = omit,
        quality: Literal["normal", "high", "very_high", "ultra_light", "ultra_dark"] | Omit = omit,
        store_media: bool | Omit = omit,
        store_preview: bool | Omit = omit,
        t38_enabled: bool | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FaxCreateResponse:
        """Send a fax.

        Files have size limits and page count limit validations. If a file
        is bigger than 50MB or has more than 350 pages it will fail with
        `file_size_limit_exceeded` and `page_count_limit_exceeded` respectively.

        **Expected Webhooks:**

        - `fax.queued`
        - `fax.media.processed`
        - `fax.sending.started`
        - `fax.delivered`
        - `fax.failed`

        Args:
          connection_id: The connection ID to send the fax with.

          from_: The phone number, in E.164 format, the fax will be sent from.

          to: The phone number, in E.164 format, the fax will be sent to or SIP URI

          client_state: Use this field to add state to every subsequent webhook. It must be a valid
              Base-64 encoded string.

          from_display_name: The `from_display_name` string to be used as the caller id name (SIP From
              Display Name) presented to the destination (`to` number). The string should have
              a maximum of 128 characters, containing only letters, numbers, spaces, and
              -\\__~!.+ special characters. If ommited, the display name will be the same as the
              number in the `from` field.

          media_name: The media_name used for the fax's media. Must point to a file previously
              uploaded to api.telnyx.com/v2/media by the same user/organization. media_name
              and media_url/contents can't be submitted together.

          media_url: The URL (or list of URLs) to the PDF used for the fax's media. media_url and
              media_name/contents can't be submitted together.

          monochrome: The flag to enable monochrome, true black and white fax results.

          preview_format: The format for the preview file in case the `store_preview` is `true`.

          quality: The quality of the fax. The `ultra` settings provides the highest quality
              available, but also present longer fax processing times. `ultra_light` is best
              suited for images, wihle `ultra_dark` is best suited for text.

          store_media: Should fax media be stored on temporary URL. It does not support media_name,
              they can't be submitted together.

          store_preview: Should fax preview be stored on temporary URL.

          t38_enabled: The flag to disable the T.38 protocol.

          webhook_url: Use this field to override the URL to which Telnyx will send subsequent webhooks
              for this fax.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "connection_id": connection_id,
                "from_": from_,
                "to": to,
                "client_state": client_state,
                "from_display_name": from_display_name,
                "media_name": media_name,
                "media_url": media_url,
                "monochrome": monochrome,
                "preview_format": preview_format,
                "quality": quality,
                "store_media": store_media,
                "store_preview": store_preview,
                "t38_enabled": t38_enabled,
                "webhook_url": webhook_url,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["contents"]])
        if files:
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/faxes",
            body=maybe_transform(body, fax_create_params.FaxCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FaxCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FaxRetrieveResponse:
        """
        View a fax

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/faxes/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FaxRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: fax_list_params.Filter | Omit = omit,
        page: fax_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FaxListResponse:
        """
        View a list of faxes

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[created_at][gte], filter[created_at][gt], filter[created_at][lte],
              filter[created_at][lt], filter[direction][eq], filter[from][eq], filter[to][eq]

          page: Consolidated pagination parameter (deepObject style). Originally: page[size],
              page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/faxes",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    fax_list_params.FaxListParams,
                ),
            ),
            cast_to=FaxListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a fax

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/faxes/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncFaxesResource(AsyncAPIResource):
    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFaxesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFaxesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFaxesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncFaxesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        connection_id: str,
        from_: str,
        to: str,
        client_state: str | Omit = omit,
        from_display_name: str | Omit = omit,
        media_name: str | Omit = omit,
        media_url: str | Omit = omit,
        monochrome: bool | Omit = omit,
        preview_format: Literal["pdf", "tiff"] | Omit = omit,
        quality: Literal["normal", "high", "very_high", "ultra_light", "ultra_dark"] | Omit = omit,
        store_media: bool | Omit = omit,
        store_preview: bool | Omit = omit,
        t38_enabled: bool | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FaxCreateResponse:
        """Send a fax.

        Files have size limits and page count limit validations. If a file
        is bigger than 50MB or has more than 350 pages it will fail with
        `file_size_limit_exceeded` and `page_count_limit_exceeded` respectively.

        **Expected Webhooks:**

        - `fax.queued`
        - `fax.media.processed`
        - `fax.sending.started`
        - `fax.delivered`
        - `fax.failed`

        Args:
          connection_id: The connection ID to send the fax with.

          from_: The phone number, in E.164 format, the fax will be sent from.

          to: The phone number, in E.164 format, the fax will be sent to or SIP URI

          client_state: Use this field to add state to every subsequent webhook. It must be a valid
              Base-64 encoded string.

          from_display_name: The `from_display_name` string to be used as the caller id name (SIP From
              Display Name) presented to the destination (`to` number). The string should have
              a maximum of 128 characters, containing only letters, numbers, spaces, and
              -\\__~!.+ special characters. If ommited, the display name will be the same as the
              number in the `from` field.

          media_name: The media_name used for the fax's media. Must point to a file previously
              uploaded to api.telnyx.com/v2/media by the same user/organization. media_name
              and media_url/contents can't be submitted together.

          media_url: The URL (or list of URLs) to the PDF used for the fax's media. media_url and
              media_name/contents can't be submitted together.

          monochrome: The flag to enable monochrome, true black and white fax results.

          preview_format: The format for the preview file in case the `store_preview` is `true`.

          quality: The quality of the fax. The `ultra` settings provides the highest quality
              available, but also present longer fax processing times. `ultra_light` is best
              suited for images, wihle `ultra_dark` is best suited for text.

          store_media: Should fax media be stored on temporary URL. It does not support media_name,
              they can't be submitted together.

          store_preview: Should fax preview be stored on temporary URL.

          t38_enabled: The flag to disable the T.38 protocol.

          webhook_url: Use this field to override the URL to which Telnyx will send subsequent webhooks
              for this fax.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "connection_id": connection_id,
                "from_": from_,
                "to": to,
                "client_state": client_state,
                "from_display_name": from_display_name,
                "media_name": media_name,
                "media_url": media_url,
                "monochrome": monochrome,
                "preview_format": preview_format,
                "quality": quality,
                "store_media": store_media,
                "store_preview": store_preview,
                "t38_enabled": t38_enabled,
                "webhook_url": webhook_url,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["contents"]])
        if files:
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/faxes",
            body=await async_maybe_transform(body, fax_create_params.FaxCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FaxCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FaxRetrieveResponse:
        """
        View a fax

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/faxes/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FaxRetrieveResponse,
        )

    async def list(
        self,
        *,
        filter: fax_list_params.Filter | Omit = omit,
        page: fax_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FaxListResponse:
        """
        View a list of faxes

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[created_at][gte], filter[created_at][gt], filter[created_at][lte],
              filter[created_at][lt], filter[direction][eq], filter[from][eq], filter[to][eq]

          page: Consolidated pagination parameter (deepObject style). Originally: page[size],
              page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/faxes",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    fax_list_params.FaxListParams,
                ),
            ),
            cast_to=FaxListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a fax

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/faxes/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class FaxesResourceWithRawResponse:
    def __init__(self, faxes: FaxesResource) -> None:
        self._faxes = faxes

        self.create = to_raw_response_wrapper(
            faxes.create,
        )
        self.retrieve = to_raw_response_wrapper(
            faxes.retrieve,
        )
        self.list = to_raw_response_wrapper(
            faxes.list,
        )
        self.delete = to_raw_response_wrapper(
            faxes.delete,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._faxes.actions)


class AsyncFaxesResourceWithRawResponse:
    def __init__(self, faxes: AsyncFaxesResource) -> None:
        self._faxes = faxes

        self.create = async_to_raw_response_wrapper(
            faxes.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            faxes.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            faxes.list,
        )
        self.delete = async_to_raw_response_wrapper(
            faxes.delete,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._faxes.actions)


class FaxesResourceWithStreamingResponse:
    def __init__(self, faxes: FaxesResource) -> None:
        self._faxes = faxes

        self.create = to_streamed_response_wrapper(
            faxes.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            faxes.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            faxes.list,
        )
        self.delete = to_streamed_response_wrapper(
            faxes.delete,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._faxes.actions)


class AsyncFaxesResourceWithStreamingResponse:
    def __init__(self, faxes: AsyncFaxesResource) -> None:
        self._faxes = faxes

        self.create = async_to_streamed_response_wrapper(
            faxes.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            faxes.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            faxes.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            faxes.delete,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._faxes.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from ..pagination_meta import PaginationMeta
from ..shared.phone_number_with_messaging_settings import PhoneNumberWithMessagingSettings

__all__ = ["MessagingListResponse"]


class MessagingListResponse(BaseModel):
    data: Optional[List[PhoneNumberWithMessagingSettings]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx._utils import parse_datetime
from telnyx.types.ai.assistants import (
    ScheduledEventResponse,
    ScheduledEventListResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestScheduledEvents:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        scheduled_event = client.ai.assistants.scheduled_events.create(
            assistant_id="assistant_id",
            scheduled_at_fixed_datetime=parse_datetime("2025-04-15T13:07:28.764Z"),
            telnyx_agent_target="telnyx_agent_target",
            telnyx_conversation_channel="phone_call",
            telnyx_end_user_target="telnyx_end_user_target",
        )
        assert_matches_type(ScheduledEventResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        scheduled_event = client.ai.assistants.scheduled_events.create(
            assistant_id="assistant_id",
            scheduled_at_fixed_datetime=parse_datetime("2025-04-15T13:07:28.764Z"),
            telnyx_agent_target="telnyx_agent_target",
            telnyx_conversation_channel="phone_call",
            telnyx_end_user_target="telnyx_end_user_target",
            conversation_metadata={"foo": "string"},
            text="text",
        )
        assert_matches_type(ScheduledEventResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.ai.assistants.scheduled_events.with_raw_response.create(
            assistant_id="assistant_id",
            scheduled_at_fixed_datetime=parse_datetime("2025-04-15T13:07:28.764Z"),
            telnyx_agent_target="telnyx_agent_target",
            telnyx_conversation_channel="phone_call",
            telnyx_end_user_target="telnyx_end_user_target",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        scheduled_event = response.parse()
        assert_matches_type(ScheduledEventResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.ai.assistants.scheduled_events.with_streaming_response.create(
            assistant_id="assistant_id",
            scheduled_at_fixed_datetime=parse_datetime("2025-04-15T13:07:28.764Z"),
            telnyx_agent_target="telnyx_agent_target",
            telnyx_conversation_channel="phone_call",
            telnyx_end_user_target="telnyx_end_user_target",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            scheduled_event = response.parse()
            assert_matches_type(ScheduledEventResponse, scheduled_event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            client.ai.assistants.scheduled_events.with_raw_response.create(
                assistant_id="",
                scheduled_at_fixed_datetime=parse_datetime("2025-04-15T13:07:28.764Z"),
                telnyx_agent_target="telnyx_agent_target",
                telnyx_conversation_channel="phone_call",
                telnyx_end_user_target="telnyx_end_user_target",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        scheduled_event = client.ai.assistants.scheduled_events.retrieve(
            event_id="event_id",
            assistant_id="assistant_id",
        )
        assert_matches_type(ScheduledEventResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.ai.assistants.scheduled_events.with_raw_response.retrieve(
            event_id="event_id",
            assistant_id="assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        scheduled_event = response.parse()
        assert_matches_type(ScheduledEventResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.ai.assistants.scheduled_events.with_streaming_response.retrieve(
            event_id="event_id",
            assistant_id="assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            scheduled_event = response.parse()
            assert_matches_type(ScheduledEventResponse, scheduled_event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            client.ai.assistants.scheduled_events.with_raw_response.retrieve(
                event_id="event_id",
                assistant_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `event_id` but received ''"):
            client.ai.assistants.scheduled_events.with_raw_response.retrieve(
                event_id="",
                assistant_id="assistant_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        scheduled_event = client.ai.assistants.scheduled_events.list(
            assistant_id="assistant_id",
        )
        assert_matches_type(ScheduledEventListResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        scheduled_event = client.ai.assistants.scheduled_events.list(
            assistant_id="assistant_id",
            conversation_channel="phone_call",
            from_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            page={
                "number": 1,
                "size": 1,
            },
            to_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(ScheduledEventListResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.ai.assistants.scheduled_events.with_raw_response.list(
            assistant_id="assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        scheduled_event = response.parse()
        assert_matches_type(ScheduledEventListResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.ai.assistants.scheduled_events.with_streaming_response.list(
            assistant_id="assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            scheduled_event = response.parse()
            assert_matches_type(ScheduledEventListResponse, scheduled_event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            client.ai.assistants.scheduled_events.with_raw_response.list(
                assistant_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        scheduled_event = client.ai.assistants.scheduled_events.delete(
            event_id="event_id",
            assistant_id="assistant_id",
        )
        assert_matches_type(object, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.ai.assistants.scheduled_events.with_raw_response.delete(
            event_id="event_id",
            assistant_id="assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        scheduled_event = response.parse()
        assert_matches_type(object, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.ai.assistants.scheduled_events.with_streaming_response.delete(
            event_id="event_id",
            assistant_id="assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            scheduled_event = response.parse()
            assert_matches_type(object, scheduled_event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            client.ai.assistants.scheduled_events.with_raw_response.delete(
                event_id="event_id",
                assistant_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `event_id` but received ''"):
            client.ai.assistants.scheduled_events.with_raw_response.delete(
                event_id="",
                assistant_id="assistant_id",
            )


class TestAsyncScheduledEvents:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        scheduled_event = await async_client.ai.assistants.scheduled_events.create(
            assistant_id="assistant_id",
            scheduled_at_fixed_datetime=parse_datetime("2025-04-15T13:07:28.764Z"),
            telnyx_agent_target="telnyx_agent_target",
            telnyx_conversation_channel="phone_call",
            telnyx_end_user_target="telnyx_end_user_target",
        )
        assert_matches_type(ScheduledEventResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        scheduled_event = await async_client.ai.assistants.scheduled_events.create(
            assistant_id="assistant_id",
            scheduled_at_fixed_datetime=parse_datetime("2025-04-15T13:07:28.764Z"),
            telnyx_agent_target="telnyx_agent_target",
            telnyx_conversation_channel="phone_call",
            telnyx_end_user_target="telnyx_end_user_target",
            conversation_metadata={"foo": "string"},
            text="text",
        )
        assert_matches_type(ScheduledEventResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.scheduled_events.with_raw_response.create(
            assistant_id="assistant_id",
            scheduled_at_fixed_datetime=parse_datetime("2025-04-15T13:07:28.764Z"),
            telnyx_agent_target="telnyx_agent_target",
            telnyx_conversation_channel="phone_call",
            telnyx_end_user_target="telnyx_end_user_target",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        scheduled_event = await response.parse()
        assert_matches_type(ScheduledEventResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.scheduled_events.with_streaming_response.create(
            assistant_id="assistant_id",
            scheduled_at_fixed_datetime=parse_datetime("2025-04-15T13:07:28.764Z"),
            telnyx_agent_target="telnyx_agent_target",
            telnyx_conversation_channel="phone_call",
            telnyx_end_user_target="telnyx_end_user_target",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            scheduled_event = await response.parse()
            assert_matches_type(ScheduledEventResponse, scheduled_event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            await async_client.ai.assistants.scheduled_events.with_raw_response.create(
                assistant_id="",
                scheduled_at_fixed_datetime=parse_datetime("2025-04-15T13:07:28.764Z"),
                telnyx_agent_target="telnyx_agent_target",
                telnyx_conversation_channel="phone_call",
                telnyx_end_user_target="telnyx_end_user_target",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        scheduled_event = await async_client.ai.assistants.scheduled_events.retrieve(
            event_id="event_id",
            assistant_id="assistant_id",
        )
        assert_matches_type(ScheduledEventResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.scheduled_events.with_raw_response.retrieve(
            event_id="event_id",
            assistant_id="assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        scheduled_event = await response.parse()
        assert_matches_type(ScheduledEventResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.scheduled_events.with_streaming_response.retrieve(
            event_id="event_id",
            assistant_id="assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            scheduled_event = await response.parse()
            assert_matches_type(ScheduledEventResponse, scheduled_event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            await async_client.ai.assistants.scheduled_events.with_raw_response.retrieve(
                event_id="event_id",
                assistant_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `event_id` but received ''"):
            await async_client.ai.assistants.scheduled_events.with_raw_response.retrieve(
                event_id="",
                assistant_id="assistant_id",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        scheduled_event = await async_client.ai.assistants.scheduled_events.list(
            assistant_id="assistant_id",
        )
        assert_matches_type(ScheduledEventListResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        scheduled_event = await async_client.ai.assistants.scheduled_events.list(
            assistant_id="assistant_id",
            conversation_channel="phone_call",
            from_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            page={
                "number": 1,
                "size": 1,
            },
            to_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(ScheduledEventListResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.scheduled_events.with_raw_response.list(
            assistant_id="assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        scheduled_event = await response.parse()
        assert_matches_type(ScheduledEventListResponse, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.scheduled_events.with_streaming_response.list(
            assistant_id="assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            scheduled_event = await response.parse()
            assert_matches_type(ScheduledEventListResponse, scheduled_event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            await async_client.ai.assistants.scheduled_events.with_raw_response.list(
                assistant_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        scheduled_event = await async_client.ai.assistants.scheduled_events.delete(
            event_id="event_id",
            assistant_id="assistant_id",
        )
        assert_matches_type(object, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.scheduled_events.with_raw_response.delete(
            event_id="event_id",
            assistant_id="assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        scheduled_event = await response.parse()
        assert_matches_type(object, scheduled_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.scheduled_events.with_streaming_response.delete(
            event_id="event_id",
            assistant_id="assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            scheduled_event = await response.parse()
            assert_matches_type(object, scheduled_event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            await async_client.ai.assistants.scheduled_events.with_raw_response.delete(
                event_id="event_id",
                assistant_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `event_id` but received ''"):
            await async_client.ai.assistants.scheduled_events.with_raw_response.delete(
                event_id="",
                assistant_id="assistant_id",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["RecordingRecordingSidJsonResponse"]


class RecordingRecordingSidJsonResponse(BaseModel):
    account_sid: Optional[str] = None

    call_sid: Optional[str] = None

    channels: Optional[Literal[1, 2]] = None

    conference_sid: Optional[str] = None

    date_created: Optional[datetime] = None

    date_updated: Optional[datetime] = None

    duration: Optional[str] = None
    """The duration of this recording, given in seconds."""

    error_code: Optional[str] = None

    price: Optional[str] = None
    """The price of this recording, the currency is specified in the price_unit field."""

    price_unit: Optional[str] = None
    """The unit in which the price is given."""

    sid: Optional[str] = None
    """Identifier of a resource."""

    source: Optional[
        Literal[
            "StartCallRecordingAPI",
            "StartConferenceRecordingAPI",
            "OutboundAPI",
            "DialVerb",
            "Conference",
            "RecordVerb",
            "Trunking",
        ]
    ] = None
    """Defines how the recording was created."""

    start_time: Optional[datetime] = None

    track: Optional[Literal["inbound", "outbound", "both"]] = None
    """The audio track to record for the call. The default is `both`."""

    uri: Optional[str] = None
    """The relative URI for this recording resource."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .global_ip_assignment_param import GlobalIPAssignmentParam

__all__ = ["GlobalIPAssignmentUpdateParams", "Body"]


class GlobalIPAssignmentUpdateParams(TypedDict, total=False):
    body: Required[Body]


class Body(GlobalIPAssignmentParam, total=False):
    pass



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["JobCreateParams", "Hyperparameters"]


class JobCreateParams(TypedDict, total=False):
    model: Required[str]
    """The base model that is being fine-tuned."""

    training_file: Required[str]
    """The storage bucket or object used for training."""

    hyperparameters: Hyperparameters
    """The hyperparameters used for the fine-tuning job."""

    suffix: str
    """Optional suffix to append to the fine tuned model's name."""


class Hyperparameters(TypedDict, total=False):
    n_epochs: int
    """The number of epochs to train the model for.

    An epoch refers to one full cycle through the training dataset. 'auto' decides
    the optimal number of epochs based on the size of the dataset. If setting the
    number manually, we support any number between 1 and 50 epochs.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .number_order_with_phone_numbers import NumberOrderWithPhoneNumbers

__all__ = ["NumberOrderRetrieveResponse"]


class NumberOrderRetrieveResponse(BaseModel):
    data: Optional[NumberOrderWithPhoneNumbers] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["Metadata"]


class Metadata(BaseModel):
    page_number: Optional[float] = None
    """Current Page based on pagination settings (included when defaults are used.)"""

    page_size: Optional[float] = None
    """
    Number of results to return per page based on pagination settings (included when
    defaults are used.)
    """

    total_pages: Optional[float] = None
    """Total number of pages based on pagination settings"""

    total_results: Optional[float] = None
    """Total number of results"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["S3ConfigurationDataParam"]


class S3ConfigurationDataParam(TypedDict, total=False):
    aws_access_key_id: str
    """AWS credentials access key id."""

    aws_secret_access_key: str
    """AWS secret access key."""

    bucket: str
    """Name of the bucket to be used to store recording files."""

    region: str
    """Region where the bucket is located."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["InsightListParams", "Page"]


class InsightListParams(TypedDict, total=False):
    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """


class Page(TypedDict, total=False):
    number: int
    """Page number (0-based)"""

    size: int
    """Number of items per page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["AvailablePhoneNumberBlockListParams", "Filter"]


class AvailablePhoneNumberBlockListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[locality], filter[country_code],
    filter[national_destination_code], filter[phone_number_type]
    """


class Filter(TypedDict, total=False):
    country_code: str
    """Filter phone numbers by country."""

    locality: str
    """Filter phone numbers by city."""

    national_destination_code: str
    """Filter by the national destination code of the number."""

    phone_number_type: Literal["local", "toll_free", "mobile", "national", "shared_cost"]
    """Filter phone numbers by number type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ...types import (
    sim_card_list_params,
    sim_card_delete_params,
    sim_card_update_params,
    sim_card_retrieve_params,
    sim_card_list_wireless_connectivity_logs_params,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.sim_card_list_response import SimCardListResponse
from ...types.sim_card_delete_response import SimCardDeleteResponse
from ...types.sim_card_update_response import SimCardUpdateResponse
from ...types.sim_card_retrieve_response import SimCardRetrieveResponse
from ...types.shared_params.sim_card_status import SimCardStatus
from ...types.sim_card_get_public_ip_response import SimCardGetPublicIPResponse
from ...types.sim_card_get_device_details_response import SimCardGetDeviceDetailsResponse
from ...types.sim_card_get_activation_code_response import SimCardGetActivationCodeResponse
from ...types.sim_card_list_wireless_connectivity_logs_response import SimCardListWirelessConnectivityLogsResponse

__all__ = ["SimCardsResource", "AsyncSimCardsResource"]


class SimCardsResource(SyncAPIResource):
    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> SimCardsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SimCardsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SimCardsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SimCardsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        include_pin_puk_codes: bool | Omit = omit,
        include_sim_card_group: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardRetrieveResponse:
        """
        Returns the details regarding a specific SIM card.

        Args:
          include_pin_puk_codes: When set to true, includes the PIN and PUK codes in the response. These codes
              are used for SIM card security and unlocking purposes. Available for both
              physical SIM cards and eSIMs.

          include_sim_card_group: It includes the associated SIM card group object in the response when present.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/sim_cards/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_pin_puk_codes": include_pin_puk_codes,
                        "include_sim_card_group": include_sim_card_group,
                    },
                    sim_card_retrieve_params.SimCardRetrieveParams,
                ),
            ),
            cast_to=SimCardRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        authorized_imeis: Optional[SequenceNotStr[str]] | Omit = omit,
        data_limit: sim_card_update_params.DataLimit | Omit = omit,
        sim_card_group_id: str | Omit = omit,
        status: SimCardStatus | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardUpdateResponse:
        """
        Updates SIM card data

        Args:
          authorized_imeis: List of IMEIs authorized to use a given SIM card.

          data_limit: The SIM card individual data limit configuration.

          sim_card_group_id: The group SIMCardGroup identification. This attribute can be <code>null</code>
              when it's present in an associated resource.

          tags: Searchable tags associated with the SIM card

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/sim_cards/{id}",
            body=maybe_transform(
                {
                    "authorized_imeis": authorized_imeis,
                    "data_limit": data_limit,
                    "sim_card_group_id": sim_card_group_id,
                    "status": status,
                    "tags": tags,
                },
                sim_card_update_params.SimCardUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardUpdateResponse,
        )

    def list(
        self,
        *,
        filter: sim_card_list_params.Filter | Omit = omit,
        filter_sim_card_group_id: str | Omit = omit,
        include_sim_card_group: bool | Omit = omit,
        page: sim_card_list_params.Page | Omit = omit,
        sort: Literal["current_billing_period_consumed_data.amount"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardListResponse:
        """
        Get all SIM cards belonging to the user that match the given filters.

        Args:
          filter:
              Consolidated filter parameter for SIM cards (deepObject style). Originally:
              filter[tags], filter[iccid], filter[status]

          filter_sim_card_group_id: A valid SIM card group ID.

          include_sim_card_group: It includes the associated SIM card group object in the response when present.

          page: Consolidated pagination parameter (deepObject style). Originally: page[number],
              page[size]

          sort: Sorts SIM cards by the given field. Defaults to ascending order unless field is
              prefixed with a minus sign.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/sim_cards",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "filter_sim_card_group_id": filter_sim_card_group_id,
                        "include_sim_card_group": include_sim_card_group,
                        "page": page,
                        "sort": sort,
                    },
                    sim_card_list_params.SimCardListParams,
                ),
            ),
            cast_to=SimCardListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        report_lost: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardDeleteResponse:
        """
        The SIM card will be decommissioned, removed from your account and you will stop
        being charged.<br />The SIM card won't be able to connect to the network after
        the deletion is completed, thus making it impossible to consume data.<br/>
        Transitioning to the disabled state may take a period of time. Until the
        transition is completed, the SIM card status will be disabling
        <code>disabling</code>.<br />In order to re-enable the SIM card, you will need
        to re-register it.

        Args:
          report_lost: Enables deletion of disabled eSIMs that can't be uninstalled from a device. This
              is irreversible and the eSIM cannot be re-registered.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/sim_cards/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"report_lost": report_lost}, sim_card_delete_params.SimCardDeleteParams),
            ),
            cast_to=SimCardDeleteResponse,
        )

    def get_activation_code(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardGetActivationCodeResponse:
        """
        It returns the activation code for an eSIM.<br/><br/> This API is only available
        for eSIMs. If the given SIM is a physical SIM card, or has already been
        installed, an error will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/sim_cards/{id}/activation_code",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardGetActivationCodeResponse,
        )

    def get_device_details(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardGetDeviceDetailsResponse:
        """
        It returns the device details where a SIM card is currently being used.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/sim_cards/{id}/device_details",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardGetDeviceDetailsResponse,
        )

    def get_public_ip(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardGetPublicIPResponse:
        """
        It returns the public IP requested for a SIM card.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/sim_cards/{id}/public_ip",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardGetPublicIPResponse,
        )

    def list_wireless_connectivity_logs(
        self,
        id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardListWirelessConnectivityLogsResponse:
        """
        This API allows listing a paginated collection of Wireless Connectivity Logs
        associated with a SIM Card, for troubleshooting purposes.

        Args:
          page_number: The page number to load.

          page_size: The size of the page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/sim_cards/{id}/wireless_connectivity_logs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    sim_card_list_wireless_connectivity_logs_params.SimCardListWirelessConnectivityLogsParams,
                ),
            ),
            cast_to=SimCardListWirelessConnectivityLogsResponse,
        )


class AsyncSimCardsResource(AsyncAPIResource):
    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSimCardsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSimCardsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSimCardsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSimCardsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        include_pin_puk_codes: bool | Omit = omit,
        include_sim_card_group: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardRetrieveResponse:
        """
        Returns the details regarding a specific SIM card.

        Args:
          include_pin_puk_codes: When set to true, includes the PIN and PUK codes in the response. These codes
              are used for SIM card security and unlocking purposes. Available for both
              physical SIM cards and eSIMs.

          include_sim_card_group: It includes the associated SIM card group object in the response when present.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/sim_cards/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "include_pin_puk_codes": include_pin_puk_codes,
                        "include_sim_card_group": include_sim_card_group,
                    },
                    sim_card_retrieve_params.SimCardRetrieveParams,
                ),
            ),
            cast_to=SimCardRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        authorized_imeis: Optional[SequenceNotStr[str]] | Omit = omit,
        data_limit: sim_card_update_params.DataLimit | Omit = omit,
        sim_card_group_id: str | Omit = omit,
        status: SimCardStatus | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardUpdateResponse:
        """
        Updates SIM card data

        Args:
          authorized_imeis: List of IMEIs authorized to use a given SIM card.

          data_limit: The SIM card individual data limit configuration.

          sim_card_group_id: The group SIMCardGroup identification. This attribute can be <code>null</code>
              when it's present in an associated resource.

          tags: Searchable tags associated with the SIM card

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/sim_cards/{id}",
            body=await async_maybe_transform(
                {
                    "authorized_imeis": authorized_imeis,
                    "data_limit": data_limit,
                    "sim_card_group_id": sim_card_group_id,
                    "status": status,
                    "tags": tags,
                },
                sim_card_update_params.SimCardUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardUpdateResponse,
        )

    async def list(
        self,
        *,
        filter: sim_card_list_params.Filter | Omit = omit,
        filter_sim_card_group_id: str | Omit = omit,
        include_sim_card_group: bool | Omit = omit,
        page: sim_card_list_params.Page | Omit = omit,
        sort: Literal["current_billing_period_consumed_data.amount"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardListResponse:
        """
        Get all SIM cards belonging to the user that match the given filters.

        Args:
          filter:
              Consolidated filter parameter for SIM cards (deepObject style). Originally:
              filter[tags], filter[iccid], filter[status]

          filter_sim_card_group_id: A valid SIM card group ID.

          include_sim_card_group: It includes the associated SIM card group object in the response when present.

          page: Consolidated pagination parameter (deepObject style). Originally: page[number],
              page[size]

          sort: Sorts SIM cards by the given field. Defaults to ascending order unless field is
              prefixed with a minus sign.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/sim_cards",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "filter_sim_card_group_id": filter_sim_card_group_id,
                        "include_sim_card_group": include_sim_card_group,
                        "page": page,
                        "sort": sort,
                    },
                    sim_card_list_params.SimCardListParams,
                ),
            ),
            cast_to=SimCardListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        report_lost: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardDeleteResponse:
        """
        The SIM card will be decommissioned, removed from your account and you will stop
        being charged.<br />The SIM card won't be able to connect to the network after
        the deletion is completed, thus making it impossible to consume data.<br/>
        Transitioning to the disabled state may take a period of time. Until the
        transition is completed, the SIM card status will be disabling
        <code>disabling</code>.<br />In order to re-enable the SIM card, you will need
        to re-register it.

        Args:
          report_lost: Enables deletion of disabled eSIMs that can't be uninstalled from a device. This
              is irreversible and the eSIM cannot be re-registered.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/sim_cards/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"report_lost": report_lost}, sim_card_delete_params.SimCardDeleteParams
                ),
            ),
            cast_to=SimCardDeleteResponse,
        )

    async def get_activation_code(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardGetActivationCodeResponse:
        """
        It returns the activation code for an eSIM.<br/><br/> This API is only available
        for eSIMs. If the given SIM is a physical SIM card, or has already been
        installed, an error will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/sim_cards/{id}/activation_code",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardGetActivationCodeResponse,
        )

    async def get_device_details(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardGetDeviceDetailsResponse:
        """
        It returns the device details where a SIM card is currently being used.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/sim_cards/{id}/device_details",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardGetDeviceDetailsResponse,
        )

    async def get_public_ip(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardGetPublicIPResponse:
        """
        It returns the public IP requested for a SIM card.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/sim_cards/{id}/public_ip",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardGetPublicIPResponse,
        )

    async def list_wireless_connectivity_logs(
        self,
        id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardListWirelessConnectivityLogsResponse:
        """
        This API allows listing a paginated collection of Wireless Connectivity Logs
        associated with a SIM Card, for troubleshooting purposes.

        Args:
          page_number: The page number to load.

          page_size: The size of the page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/sim_cards/{id}/wireless_connectivity_logs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    sim_card_list_wireless_connectivity_logs_params.SimCardListWirelessConnectivityLogsParams,
                ),
            ),
            cast_to=SimCardListWirelessConnectivityLogsResponse,
        )


class SimCardsResourceWithRawResponse:
    def __init__(self, sim_cards: SimCardsResource) -> None:
        self._sim_cards = sim_cards

        self.retrieve = to_raw_response_wrapper(
            sim_cards.retrieve,
        )
        self.update = to_raw_response_wrapper(
            sim_cards.update,
        )
        self.list = to_raw_response_wrapper(
            sim_cards.list,
        )
        self.delete = to_raw_response_wrapper(
            sim_cards.delete,
        )
        self.get_activation_code = to_raw_response_wrapper(
            sim_cards.get_activation_code,
        )
        self.get_device_details = to_raw_response_wrapper(
            sim_cards.get_device_details,
        )
        self.get_public_ip = to_raw_response_wrapper(
            sim_cards.get_public_ip,
        )
        self.list_wireless_connectivity_logs = to_raw_response_wrapper(
            sim_cards.list_wireless_connectivity_logs,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._sim_cards.actions)


class AsyncSimCardsResourceWithRawResponse:
    def __init__(self, sim_cards: AsyncSimCardsResource) -> None:
        self._sim_cards = sim_cards

        self.retrieve = async_to_raw_response_wrapper(
            sim_cards.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            sim_cards.update,
        )
        self.list = async_to_raw_response_wrapper(
            sim_cards.list,
        )
        self.delete = async_to_raw_response_wrapper(
            sim_cards.delete,
        )
        self.get_activation_code = async_to_raw_response_wrapper(
            sim_cards.get_activation_code,
        )
        self.get_device_details = async_to_raw_response_wrapper(
            sim_cards.get_device_details,
        )
        self.get_public_ip = async_to_raw_response_wrapper(
            sim_cards.get_public_ip,
        )
        self.list_wireless_connectivity_logs = async_to_raw_response_wrapper(
            sim_cards.list_wireless_connectivity_logs,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._sim_cards.actions)


class SimCardsResourceWithStreamingResponse:
    def __init__(self, sim_cards: SimCardsResource) -> None:
        self._sim_cards = sim_cards

        self.retrieve = to_streamed_response_wrapper(
            sim_cards.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            sim_cards.update,
        )
        self.list = to_streamed_response_wrapper(
            sim_cards.list,
        )
        self.delete = to_streamed_response_wrapper(
            sim_cards.delete,
        )
        self.get_activation_code = to_streamed_response_wrapper(
            sim_cards.get_activation_code,
        )
        self.get_device_details = to_streamed_response_wrapper(
            sim_cards.get_device_details,
        )
        self.get_public_ip = to_streamed_response_wrapper(
            sim_cards.get_public_ip,
        )
        self.list_wireless_connectivity_logs = to_streamed_response_wrapper(
            sim_cards.list_wireless_connectivity_logs,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._sim_cards.actions)


class AsyncSimCardsResourceWithStreamingResponse:
    def __init__(self, sim_cards: AsyncSimCardsResource) -> None:
        self._sim_cards = sim_cards

        self.retrieve = async_to_streamed_response_wrapper(
            sim_cards.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            sim_cards.update,
        )
        self.list = async_to_streamed_response_wrapper(
            sim_cards.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            sim_cards.delete,
        )
        self.get_activation_code = async_to_streamed_response_wrapper(
            sim_cards.get_activation_code,
        )
        self.get_device_details = async_to_streamed_response_wrapper(
            sim_cards.get_device_details,
        )
        self.get_public_ip = async_to_streamed_response_wrapper(
            sim_cards.get_public_ip,
        )
        self.list_wireless_connectivity_logs = async_to_streamed_response_wrapper(
            sim_cards.list_wireless_connectivity_logs,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._sim_cards.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["UsageReportGetOptionsParams"]


class UsageReportGetOptionsParams(TypedDict, total=False):
    product: str
    """Options (dimensions and metrics) for a given product.

    If none specified, all products will be returned.
    """

    authorization_bearer: str
    """Authenticates the request with your Telnyx API V2 KEY"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .verification import (
    VerificationResource,
    AsyncVerificationResource,
    VerificationResourceWithRawResponse,
    AsyncVerificationResourceWithRawResponse,
    VerificationResourceWithStreamingResponse,
    AsyncVerificationResourceWithStreamingResponse,
)
from .messaging_tollfree import (
    MessagingTollfreeResource,
    AsyncMessagingTollfreeResource,
    MessagingTollfreeResourceWithRawResponse,
    AsyncMessagingTollfreeResourceWithRawResponse,
    MessagingTollfreeResourceWithStreamingResponse,
    AsyncMessagingTollfreeResourceWithStreamingResponse,
)

__all__ = [
    "VerificationResource",
    "AsyncVerificationResource",
    "VerificationResourceWithRawResponse",
    "AsyncVerificationResourceWithRawResponse",
    "VerificationResourceWithStreamingResponse",
    "AsyncVerificationResourceWithStreamingResponse",
    "MessagingTollfreeResource",
    "AsyncMessagingTollfreeResource",
    "MessagingTollfreeResourceWithRawResponse",
    "AsyncMessagingTollfreeResourceWithRawResponse",
    "MessagingTollfreeResourceWithStreamingResponse",
    "AsyncMessagingTollfreeResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["TelephonySettings"]


class TelephonySettings(BaseModel):
    default_texml_app_id: Optional[str] = None
    """Default Texml App used for voice calls with your assistant.

    This will be created automatically on assistant creation.
    """

    supports_unauthenticated_web_calls: Optional[bool] = None
    """
    When enabled, allows users to interact with your AI assistant directly from your
    website without requiring authentication. This is required for FE widgets that
    work with assistants that have telephony enabled.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .messaging_error import MessagingError

__all__ = [
    "MessageCancelScheduledResponse",
    "Cost",
    "CostBreakdown",
    "CostBreakdownCarrierFee",
    "CostBreakdownRate",
    "From",
    "Media",
    "To",
]


class Cost(BaseModel):
    amount: Optional[str] = None
    """The amount deducted from your account."""

    currency: Optional[str] = None
    """The ISO 4217 currency identifier."""


class CostBreakdownCarrierFee(BaseModel):
    amount: Optional[str] = None
    """The carrier fee amount."""

    currency: Optional[str] = None
    """The ISO 4217 currency identifier."""


class CostBreakdownRate(BaseModel):
    amount: Optional[str] = None
    """The rate amount applied."""

    currency: Optional[str] = None
    """The ISO 4217 currency identifier."""


class CostBreakdown(BaseModel):
    carrier_fee: Optional[CostBreakdownCarrierFee] = None

    rate: Optional[CostBreakdownRate] = None


class From(BaseModel):
    carrier: Optional[str] = None
    """The carrier of the receiver."""

    line_type: Optional[Literal["Wireline", "Wireless", "VoWiFi", "VoIP", "Pre-Paid Wireless", ""]] = None
    """The line-type of the receiver."""

    phone_number: Optional[str] = None
    """
    Sending address (+E.164 formatted phone number, alphanumeric sender ID, or short
    code).
    """


class Media(BaseModel):
    content_type: Optional[str] = None
    """The MIME type of the requested media."""

    sha256: Optional[str] = None
    """The SHA256 hash of the requested media."""

    size: Optional[int] = None
    """The size of the requested media."""

    url: Optional[str] = None
    """The url of the media requested to be sent."""


class To(BaseModel):
    carrier: Optional[str] = None
    """The carrier of the receiver."""

    line_type: Optional[Literal["Wireline", "Wireless", "VoWiFi", "VoIP", "Pre-Paid Wireless", ""]] = None
    """The line-type of the receiver."""

    phone_number: Optional[str] = None
    """Receiving address (+E.164 formatted phone number or short code)."""

    status: Optional[
        Literal[
            "scheduled",
            "queued",
            "sending",
            "sent",
            "cancelled",
            "expired",
            "sending_failed",
            "delivery_unconfirmed",
            "delivered",
            "delivery_failed",
        ]
    ] = None
    """The delivery status of the message."""


class MessageCancelScheduledResponse(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    completed_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the message was finalized."""

    cost: Optional[Cost] = None

    cost_breakdown: Optional[CostBreakdown] = None
    """Detailed breakdown of the message cost components."""

    direction: Optional[Literal["outbound"]] = None
    """The direction of the message.

    Inbound messages are sent to you whereas outbound messages are sent from you.
    """

    encoding: Optional[str] = None
    """Encoding scheme used for the message body."""

    errors: Optional[List[MessagingError]] = None
    """
    These errors may point at addressees when referring to unsuccessful/unconfirmed
    delivery statuses.
    """

    from_: Optional[From] = FieldInfo(alias="from", default=None)

    media: Optional[List[Media]] = None

    messaging_profile_id: Optional[str] = None
    """Unique identifier for a messaging profile."""

    organization_id: Optional[str] = None
    """The id of the organization the messaging profile belongs to."""

    parts: Optional[int] = None
    """Number of parts into which the message's body must be split."""

    received_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the message request was received."""

    record_type: Optional[Literal["message"]] = None
    """Identifies the type of the resource."""

    sent_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the message was sent."""

    subject: Optional[str] = None
    """Subject of multimedia message"""

    tags: Optional[List[str]] = None
    """Tags associated with the resource."""

    tcr_campaign_billable: Optional[bool] = None
    """Indicates whether the TCR campaign is billable."""

    tcr_campaign_id: Optional[str] = None
    """The Campaign Registry (TCR) campaign ID associated with the message."""

    tcr_campaign_registered: Optional[str] = None
    """The registration status of the TCR campaign."""

    text: Optional[str] = None
    """Message body (i.e., content) as a non-empty string.

    **Required for SMS**
    """

    to: Optional[List[To]] = None

    type: Optional[Literal["SMS", "MMS"]] = None
    """The type of message."""

    valid_until: Optional[datetime] = None
    """
    Message must be out of the queue by this time or else it will be discarded and
    marked as 'sending_failed'. Once the message moves out of the queue, this field
    will be nulled
    """

    webhook_failover_url: Optional[str] = None
    """
    The failover URL where webhooks related to this message will be sent if sending
    to the primary URL fails.
    """

    webhook_url: Optional[str] = None
    """The URL where webhooks related to this message will be sent."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["DocumentLinkListParams", "Filter", "Page"]


class DocumentLinkListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter for document links (deepObject style).

    Originally: filter[linked_record_type], filter[linked_resource_id]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """


class Filter(TypedDict, total=False):
    linked_record_type: str
    """The linked_record_type of the document to filter on."""

    linked_resource_id: str
    """The linked_resource_id of the document to filter on."""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .pagination_meta import PaginationMeta

__all__ = ["ManagedAccountListResponse", "Data"]


class Data(BaseModel):
    id: str
    """Uniquely identifies the managed account."""

    api_user: str
    """The manager account's email, which serves as the V1 API user identifier"""

    created_at: str
    """ISO 8601 formatted date indicating when the resource was created."""

    email: str
    """The managed account's email."""

    manager_account_id: str
    """The ID of the manager account associated with the managed account."""

    record_type: Literal["managed_account"]
    """Identifies the type of the resource."""

    updated_at: str
    """ISO 8601 formatted date indicating when the resource was updated."""

    managed_account_allow_custom_pricing: Optional[bool] = None
    """
    Boolean value that indicates if the managed account is able to have custom
    pricing set for it or not. If false, uses the pricing of the manager account.
    Defaults to false. There may be time lag between when the value is changed and
    pricing changes take effect.
    """

    organization_name: Optional[str] = None
    """The organization the managed account is associated with."""

    rollup_billing: Optional[bool] = None
    """
    Boolean value that indicates if the billing information and charges to the
    managed account "roll up" to the manager account. If true, the managed account
    will not have its own balance and will use the shared balance with the manager
    account. This value cannot be changed after account creation without going
    through Telnyx support as changes require manual updates to the account ledger.
    Defaults to false.
    """


class ManagedAccountListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None
